{******************************************************}
{                                                      }
{    TLsNumericEdit - a numeric value Edit component   }
{                     for Delphi versions 3 to 7       }
{    ===============================================   }
{    Version        - 1.00                             }
{    Author         - Leo D. Shih                      }
{    Mailto         - ldshih@ecn.ab.ca                 }
{                                                      }
{******************************************************}


unit LsNumericEdit;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TLsNumericEdit = class(TCustomEdit)
  private
    FMask: string;
    FValue: double;
    FAddChar: string;
    FFocused: Boolean;
    FNormalColor,
      FFocusedColor: TColor;

    function GetValueAsInt: LongInt;
    procedure SetValueAsInt(aValue: LongInt);
    procedure SetValue(Value: double);
    procedure SetNormalColor(Value: TColor);
    procedure SetFocusedColor(Value: TColor);

  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
    procedure KeyPress(var Key: Char); override;
    procedure KeyDown(var Key: word; Shift: TShiftstate); override;
    procedure Loaded; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetFocused(Value: Boolean);

  published
    property NormalColor: TColor read FNormalColor write SetNormalColor default clWindow;
    property FocusedColor: TColor read FFocusedColor write SetFocusedColor default clInfoBk; //clYellow;
    property Mask: string read FMask write FMask;
    property Value: double read FValue write SetValue;
    property ValueAsInt: longInt read GetValueAsInt write SetValueAsInt;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property Hint;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;

  end;

function GetNumber(Value: string; var Number: double): boolean;

procedure Register;

implementation

function GetNumber(Value: string; var Number: double): boolean;
var
  i: Integer;
  NumStr: string;
begin
  Result := true;
  for i := 1 to Length(Value) do
  begin
    case Value[i] of
      '0'..'9': NumStr := NumStr + Value[i];
      '+', '-': begin
          if i = 1 then
            NumStr := NumStr + Value[i]
          else
            Result := false;
        end;
    else
      if Value[i] = DecimalSeparator then
        NumStr := NumStr + Value[i]
      else if not (Value[i] = ThousandSeparator) then
        Result := false;
    end; //case
    if NumStr = '' then NumStr := '0';
  end;
  Number := StrToFloat(NumStr);
end; //GetNumber

constructor TLsNumericEdit.Create(AOwner: TComponent);
begin
  inherited;
  Width := 100;
  Value := 0;
  FMask := ',##0.00';
  FAddChar := '';
  FFocused := False;
  Enabled := True;
  FNormalColor := clWindow;
  FFocusedColor := clInfoBk;
end; //Create

destructor TLsNumericEdit.Destroy;
begin
  inherited Destroy;
end; //Destroy

procedure TLsNumericEdit.SetNormalColor(Value: TColor);
begin
  if FNormalColor <> Value then
    FNormalColor := Value;
  if Enabled = True then
    inherited Color := FNormalColor;
end; //SetNormalColor

procedure TLsNumericEdit.SetFocusedColor(Value: TColor);
begin
  if FFocusedColor <> Value then
    FFocusedColor := Value;
  if FFocused then
    inherited color := Value;
end; //SetFocusedColor

procedure TLsNumericEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then
  begin
    FFocused := Value;
    if FFocused then
      SetFocusedColor(FFocusedColor)
    else
      SetNormalColor(FNormalColor);
  end;
end; //SetFocused

procedure TLsNumericEdit.Loaded;
begin
  inherited Loaded;
  if Enabled then
    inherited Color := FNormalColor
  else
    inherited Color := clBtnFace;
end; //Loaded

procedure TLsNumericEdit.SetValue(Value: double);
begin
  FValue := Value;
  Text := FormatFloat(FMask, FValue);
end; //SetValue

function TLsNumericEdit.GetValueAsInt: LongInt;
begin
  Result := Trunc(Value);
end;  //GetValueAsInt

procedure TLsNumericEdit.SetValueAsInt(aValue: LongInt);
begin
  SetValue(aValue);
end;  //SetValueAsInt

procedure TLsNumericEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style + ES_RIGHT;
end; //CreateParams

procedure TLsNumericEdit.CMEnter(var Message: TCMEnter);
var
  S: string;
  isEmpty: Boolean;
  oldLen: integer;
begin
  SetFocused(True);
  if AutoSelect and  not (csLButtonDown in ControlState) then
  begin
    Text := FormatFloat(Mask, FValue); //FloatToStr(FValue);
    SelectAll;
  end;
  S := inherited Text;
  oldLen := Length(S);
  isEmpty := (oldLen = 0) or (S = '-') or (Value = 0);
  if isEmpty then Text := '';
  inherited;
end; //CMEnter

procedure TLsNumericEdit.CMExit(var Message: TCMExit);
begin
  SetFocused(False);
  inherited Color := FNormalColor;
  if Text = '' then
    Text := '0';
  if GetNumber(Text, FValue) then
    Text := FormatFloat(Mask, FValue);
  inherited DoExit;
end; //CMExit

procedure TLsNumericEdit.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if (Button = mbLeft) or (ssLeft in Shift) then
    if AutoSelect then
      SelectAll;
  inherited MouseDown(Button, Shift, X, Y);
end;  //mousedown

procedure TLsNumericEdit.KeyPress(var Key: Char);
var
  isAbort: boolean;
  i: Integer;
begin

  isAbort := true;
  case Key of
    '0'..'9': isAbort := false;
  end;
  if Key = DecimalSeparator then
    isAbort := false;

  if Ord(Key) = 8 then //Backspace
    isAbort := false;

//  if Ord(Key) = VK_RETURN then
//    Key := #9;  //3.0

  i := GetSelStart;
  if (i = 0) and ((Key = '-') or (Key = '+')) then
    isAbort := false;

  if isAbort then
  begin
    Beep;
    Abort;
  end;
end; //KeyPress


procedure TLsNumericEdit.KeyDown(var Key: word; Shift: TShiftstate);
begin
  if Key = VK_RETURN then
    PostMessage(Handle, WM_KEYDOWN, VK_TAB, 0);
  inherited KeyDown(key, shift);
end;

procedure Register;
begin
  RegisterComponents('LsComp', [TLsNumericEdit]);
end;

end.

